# YASARA MOVIE
# TOPIC:    1. General
# TITLE:    1.5. Virtual reality (Android)
# REQUIRES: Dynamics 18.1.1
# AUTHOR:   Elmar Krieger
# LICENSE:  GPL
# DESCRIPTION: A tutorial explaining how to use the YASARA Virtual Reality Workstation on Android. It covers setting up the VR headset, molecular graphics, interactive MD and drug design.

# TODO: Menu not clickable during MD

# Set to 0 to skip most of the first VR headset setup instructions
setupshown=1
# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
OnError Continue
BallStickRadius 50,50
StickRadius 50
AtomSize 100
LightSource Ambience=085,Shadow=((1-Android)*50)
Fog Density=100,Range=Static,DisMin=0%,DisMax=100%
CoordSys Left
LabelPar OnTop=0,Shrink=1,Fog=0
PolygonPar Reflection=0
Console off
HUD off
ColorBG Black

#
# Fly through the YASARA logo 
#
YASARALogo:
Clear
UserInput off
if Android
  # In Android we always want to watch this in VR mode
  Stereo VR
StereoPar EyeDis=60,ProTru=0,RPFMax=2
LoadPNG cubic
ShowImage cubic,priority=0
# Load restriction enzyme PVuII
pvuobj = LoadPDB pvu2
Style Ball
HideAtom SideChain NucAcid
for col in 'H,320','C,330','N,Blue','O,340','S,330','P,340'
  ColorAtom Obj (pvuobj) Element (col)
# Create YASARA logo frame around it
# The logo frame created by ShowLogoObj 189 A x 68.4 A tall and 9 A thick. 
lenx=189.
leny=68.4e0
lenz=9.
# Build the logo frame from individual Xenon atoms to enable collisions during MD
ShowLogoObj (pvuobj),Letters=No
# Create the 'YASARA' letters
textlist="YAS  RA","V"
for i=1 to 2
  txtobj(i) = MakeTextObj YASARA,Width=200,Height=25
  PosText 50%,0%,justify=center
  Font Baikal,Height=100%,Color=White,Depth=2,DepthCol=404040
  Print (textlist(i))
PrintCon
MoveObj (txtobj2),X=8
MoveObj (join txtobj),Y=-5,Z=-45
Move Y=40,Z=180
AutoRotateObj pvu2,X=-1
RotateObj (txtobj2),X=20
AutoRotateObj (txtobj2),X=1
# Animate the startup screen with 1CRN flying back and forth
LoadPDB 1crn
ShowSecStrObj 1crn,Ribbon
HideObj 1crn
AutoRotateObj 1crn,Y=2
# Take snapshot here, Eliza!
if setupshown
  # Describe complete VR headset setup
  textlist="Make sure the mouse pointer is visible, usually by holding a button for a while.",
           "Welcome to Yet Another Scientific Virtual Reality Application!",
           "Note that the top menu line becomes visible if you move the mouse pointer up.",
           "If the mouse pointer is shimmering strangely, its size differs between the eyes.",
           "In this case click Options > Input devices > Mouse pointer size.",
           "First, you need to make sure that the letter 'A' in the middle hardly moves...",
           "..when you open and close your eyes alternatingly. Use the wheel on the headset.",
           "Now look at this message in the front and the alphabet in the back alternatingly.",
           "If this change of focus causes too much strain, reduce the stereoscopic effect:",
           "Click Window > Stereo > Parameters > Distance between, and reduce the value.",
           "Look at the line with letters from P to A to P. You should see the same letter...",
           "...on both edges with both eyes. If not, click Win > Stereo > Params > Lens shift.",
           "If you did that, you now have to center letter 'A' again with the headset wheel.",
           "You can shift the GUI vertically at Window>Stereo>Parameters>Vertical shift.",
           "Without a Celexon VRG1, adjust the lens distortion and make sure that the...",
           "..white lines are parallel: Click Window>Stereo>Parameters>Lens distortion.",
           "Head-tracking becomes active as soon as there is no static background.",
           "Now the static image has been replaced with a skysphere, you can look around.",
           "Get ready to fly through the YASARA logo. Look back when you're through..."
else
  # Shortcut for demos 
  textlist="Make sure the mouse pointer is visible, usually by holding a button for a while.",
           "Note that the top menu line becomes visible if you move the mouse pointer up.",
           "Welcome to Yet Another Scientific Virtual Reality Application!",
           "Get ready to fly through the YASARA logo. Look back when you're through..."
for i=1 to count textlist  
  ShowMessage (textlist(i))
  ShowButton Continue
  PosObj 1crn,0,-7,1230
  while 1
    for j=0.0 to 180.0
      MoveObj 1crn,Z=(-cos j*21)
      Wait 2
      if ContinueButton
        break 2
  if (setupshown and i==17) or (!setupshown and i==1)
    ShowOriSkySphere
    AutoMoveImage cubic,Alpha=0,Steps=150
    Wait 150
    DelImage cubic
    Stereo Center
# Fly through the YASARA logo
AutoPosObj 1crn,Z=-20,Steps=100
DelObj 1crn
HideMessage
AutoPos 56,-7.4,16.5,Steps=1200
AutoPos Z=-200,Steps=1000
AutoPos Z=-1200,Steps=200
DelObj !SkySphere


# 
# Show MD simulation of hot air
#
MDofAir:
StartSection
if Android
  Antialias 1
Stereo Center
cellsize=100
Cell (cellsize),(cellsize),(cellsize)
# Fill it with air
oxyobj = BuildSMILES "O=O"
nitobj = BuildSMILES "N#N"
co2obj = BuildSMILES "O=C=O"
NameObj (oxyobj),Oxygen
NameObj (nitobj),Nitrogen
NameObj (co2obj),CO2
density=2
FillCellObj (nitobj),Copies=(77*density)
FillCellObj (co2obj),Copies=( 3*density)
Style Ball
Pos Z=-40
AutoPos Z=350,Steps=200
# Force field parameters
ForceField AMBER14
Cutoff 5
Longrange None
Boundary Wall
TimeStep 4,3
Temp 2000K
Brake 6000
textlist="Just in front of you is a box with air. Unfortunately the 'front' is not stable.",
         "With Android headtracking, you may have to center the view from time to time.",
         "To center the view, either click Window > Stereo > Center VR device...",
         "...or blindly press the ~ or ^ key left to '1' on the keyboard (easy to find).",
         "Now use the mouse just as you always do in YASARA to move the box around.",
         "Hold left button to rotate, right and both buttons to move along Z and XY.",
         "Looking at the box with air, there is a bug: mostly nitrogens, but only 1 oxygen.",
         "Click Simulation > Fill cell with > Copies, select 'Oxygen' and 40 copies.",
         "Now let's start the simulation: Click Simulation > Simulator > On or press F12",
         "Choose a slow molecule and try to left-click it to mark it. To aim correctly...",
         "...you have to mentally focus on your right eye or simply close the left eye.",
         "With a bit of practice, you will soon manage to click without closing an eye.",
         "Interactive MD of course also works in VR mode. To pull the marked molecule...",
         "...blindly hold down the Ctrl key and keep the right mouse button pressed.",
         "Let's play a game: try to click as many atoms as possible within 15 picosecs."
for i=1 to count textlist
  ShowMessage (textlist(i))
  Wait ContinueButton
  if i==8
    if Atoms==(77*2+3*3)*density+2
      FillCellObj Oxygen,Copies=40,BumpSum=1.0,RandomOri=0,DisMin=0.0
    TempCtrl SteepDes
    Sim On
    Wait SpeedMax<3000
    Wait 20
    Sim Off
    TempCtrl Rescale  
    if Android
      Antialias 0
  if i==9
    Sim On
# The video game for fast clickers
LabelPar Font=Arial
while 1
  # Ready to start...
  Color Element
  Sim On
  ShowMessage "The faster the molecule you click, the higher the score. Click 'Continue' to start."
  Wait ContinueButton
  HideMessage
  Time 0
  score=0.
  lastt=0
  MarkAtom None
  # Main game loop
  do
    Wait 5
    atm = MarkAtom
    if atm
      speed = SpeedRes Atom (atm)
      score=score+speed/1000
      ColorRes Atom (atm),Yellow
      Wait 5
      HideRes Atom (atm)
      MarkAtom None
    t = Time
    t=15-t/1000
    if t!=lastt or score!=lastscore
      # Update Score and Time label only when needed
      lastt=t
      lastscore=score
      UnlabelAll
      LabelAll 'Score (0.0+score)',Height=0.5,Color=Yellow,X=-2.5,Y=3
      LabelAll 'Time (t)',Height=0.5,Color=Yellow,X=2.5,Y=3
  while t>0
  ShowMessage 'Your final score is (0.0+score). Molecules you hit are colored yellow.'
  ShowAll
  Wait ContinueButton
  ShowMessage 'Press the left button to play again, and the right button to continue...'
  # Wait until no button is pressed
  while LeftButton or RightButton
    Wait 1
  # Make the choice
  while 1
    if LeftButton
      # Continue game loop
      continue 2
    if RightButton
      # Break out of game loop
      break 2
    Wait 1
AutoPos Z=-80,Steps=150

# 
# Show GroEL/ES
#
GroELES:
StartSection
Stereo Center
UserInput off
StereoPar RPFMax=10
# Load 1AON
Style Ball
obj=LoadPDB 1aon
for mol='A' to 'U'
  ColorMol (mol),((ord mol-65)*17)
PosObj (obj),0,0,0
RotateAtom Obj (obj),X=180
# Split into molecule objects
objlist() = SplitObj (obj)
# Join the Mg atoms and ATP molecules to their objects
for i=1 to 7
  JoinObj (objlist(i+21)),(objlist(i))
  JoinObj (objlist(i+28)),(objlist(i))
# Get the positions of the objects
poslist() = PosObj 1aon?
# Move them outwards and give them a random rotation
scale=5
for i=1 to Objects-1
  obj=objlist(i)
  PosObj (obj),(poslist(i*3-2)*scale),(poslist(i*3-1)*scale),(poslist(i*3)*scale)
  AutoRotateObj (obj),(rnd 1-0.5),(rnd 1-0.5),(rnd 1-0.5)
for text in "The proteins rotating around you are the 21 building blocks of GroEL/ES.",
            "GroEL/ES is a chaperonin that helps folding proteins by encapsulating them.",
            "Look around and try to find them all. They are arranged in three rings.",
            "If you press 'Continue', you will be encapsulated yourself. Get ready to fold..."
  ShowMessage (text)
  Wait ContinueButton
# Encapsulate the viewer
HideMessage
for i=1 to Objects-1
  obj=objlist(i)
  steps=1000+rnd 1000
  AutoPosOriObj (obj),(poslist(i*3-2)),(poslist(i*3-1)),(poslist(i*3)),0,0,0,Steps=(steps),Wait=No
Wait 2000
UserInput on
# Calculate ambient lighting for assembled GroEL/ES
#JoinObj 3-(Objects),2
textlist="Can you blindly press keys <F1> to <F8>? (Holding the Fn key may be needed!)",
         "If not, you can use the virtual keyboard by moving the cursor to the...",
         "far left or right side, and clicking on grey box that becomes visible.",
         "Use the left if you want to press multiple keys, and the right for a single key.",
         "Press the top right 'Hide' key to hide the virtual keyboard again.",
         "Note that the space filling Ball-style is the fastest. If you pick a slow style..",
         "...the frame rate may drop till the VR experience breaks and you get dizzy.",
         "Best disable head-tracking as soon as drawing a new frame takes too long.",
         "Click Window > Stereo > Parameters and choose 2 refresh cycles on the right.",
         "The screen refresh is 60 times/sec, so the VR limit is now 60/2 = 30 frames/sec.",
         "If the limit is exceeded for 50 frames in a row, head-tracking is disabled.",
         "Press 'Continue' to proceed to the next part of this tutorial..."
for i=1 to count textlist
  ShowMessage (textlist(i))
  Wait ContinueButton
  if i==5
    Style Ribbon,Stick
    UserInput on
HideMessage
AutoPos 0,0,1500,Steps=300 


#
# Visualize HIV protease
#
VisualizeHIVProtease:
StartSection
Style Ribbon,Off,BallStick
textcom="After all the VR fun, let's finish with a more scientific example: drug design",-1,'pass',
        "What follows is a shortened VR version of the help movie '4.5. Drug design'.",-1,'pass',
        "Click File>Load>PDB file and pick 3i7e to load HIV protease with TMC-126.",0,'LoadPDB 3i7e',
        "Since X-ray structures usually don't resolve hydrogens, these need to be added.",-1,'pass',
        "Click Edit > Add > Hydrogens to Object, double-click 3i7e.",1655,'AddHyd',
        "To visualize interactions, you would click View>Style scene>Ribbon&ligand.",-1,'pass',
        "This option does not work in movies, so it will be done automatically now...",-4,'LoadSce 3i7e',
        "Green lines are hydrophobic interactions, style them at Effects>Point&line app.",-1,'pass',
        "Let's create a view that shows only the active site.",-1,'pass',
        "Click View > Duplicate, type 'Active site', blindly or with the virtual keyboard.",-4,'LoadSce 3i7eviews',
        "There are two different ways to select surrounding residues.",-1,'pass',
        "1st those within a distance from the ligand center: click any atom in the ligand..",-1,'pass',
        "...right-click on it and Select>in sphere around>Res, pull to 10 A (hold <Tab>).",-1,'pass',
        "And alternatively those residues within a distance from any ligand atom:",-1,'pass',
        "Click Edit > Select > New > Residue, select 'all' in the right list and as manual selection type: with distance<5 from djr",-1,'SelectRes all with distance<5 from DJR',
        "By default, the 'Ribbon & Ligand' style shows only atoms involved in interactions.",-1,'pass',
        "To see all atoms around,right-click on a selected atom and choose Show>Atoms.",-1,'pass',
        "Click View > Hide atoms > Aliphatic hydrogens to make the view less crowded.",-1,'HideAtom Element H with bond to Element C',
        "The water molecules are not really needed, so click Edit > Delete > Waters.",-1,'DelWater',
        "In the next step, let's create a partial surface...",-1,'pass',
        "Click Edit > Add > To surface environment:Residue. Delete the 'with distance<5..' you typed before and double-click 'Protein'.",-1,'AddEnvRes Protein',
        "Click View>Show surface of>Residue,choose 'Selected' and (with <Ctrl> held) 'Protein', set color 'AtomCol', inside color 'Green'.",-1,'ShowSurfRes Protein Selected,Type=Molecular,Update=Dynamic,OutCol=AtomCol,OutAlpha=85,InCol=Green,InAlpha=60,Specular=Yes',
        "Now the surface has opacity 85 when seen from outside, and 60 from inside.",-1,'pass',
        "To get rid of the rest, click Edit > Select > Invert, now the rest is selected.",-1,'pass',
        "Right-click on the now selected ribbon and choose Hide > Secondary structure.",-1,'HideSecStrRes all with distance>5 from DJR',
        "To predict the affinity of this ligand, check help movie 4.5 on your PC.",-1,'pass'
ShowTutorial textcom
UnselectAll
AutoMoveObj !SkySphere,Z=-1
Wait 100

#
# Improve ligand
#
ImproveLigand:
StartSection
# Includes "RemoveEnvRes all with distance>10 from DJR" to boost performance on Android VR
LoadSce building1
ShowOriSkySphere
LabelRes Pro 81 Mol B,Format="RESName RESNUM",Height=0.5
LabelRes Ile 50 Mol A,Format="RESName RESNUM",Height=0.5,X=-0.5,Y=-1,Z=-1
ShowMessage "To improve the ligand, we will increase its hydrophobic interactions..."
Wait ContinueButton
ShowMessage "...by squeezing a phenyl ring between Ile 50 and Pro 81."
for i=85 to 40 step -1
  ColorSurfObj Receptor,OutAlpha=(i),InAlpha=60
  Wait 1
ShowArrow Start=AtAtom,2840,End=AtAtom,798,Radius=0.5,Heads=1,Color=Magenta
LabelDis 2840,798,Format="DIS A",Height=0.7,Z=-0.5
Wait ContinueButton
ForceField NOVA,SetPar=Yes
Boundary Periodic
SimSteps 1
textcom="Click View>Show atoms>Residue, double-click 'Djr' to bring hydrogens back.",-1,'ShowRes DJR',
        "This approach has been described by Arun K. Ghosh et al., let's try it...",-1,'pass',
        "Click the benzene-icon at the top to activate the molecular builder...",-1,'ColorAtom H13 Res DJR,Yellow',
        "Click on the yellow hydrogen, then on the Phenyl group in the builder.",3206,'SwapAtom 3161,Phenyl,AttachPoint=1',
        "The new ring strongly bumps into Ile 50, so the rings cannot be co-planar.",-1,'pass',
        "To change a dihedral, mark it: click atom 1, hold Ctrl and click the other 3.",-5,'MarkAtom 3172,3160,3161,3162',
        "Right-click on the atom marked white,select Geometry>Dihedral,move it to -50.",-1,'Dihedral 3172,3160,3161,3162,set=-50',
        "Finally, Ghosh et al. added an orto-methoxy group...",-1,'ColorAtom 3163,Yellow',
        "Click on the yellow hydrogen,then click the Methoxy-group in the builder.",3216,'SwapAtom 3163,Methoxy',
        "To get rid of the clashes, an energy minimization is needed.",-6,'pass',
        "Click Options > Choose experiment > Energy minimization.",-4,'LoadSce building2',
        "Note how the distance between Ile 50 and Pro 81 increased to 7 A...",-1,'pass',
        "...and the surface now shows a cleft that accomodated the phenyl.",-1,'HideAtom Element H with bond to Element C'
ShowTutorial textcom

#
# The End
#
TheEnd:
StartSection
UserInput off
# Load and replicate the Klingon warbirds
LoadYOb pyromellitimide
Style Ball
molecules=80
for i=2 to molecules
  DuplicateObj 2
# Skysphere is object 1
for i=2 to molecules+1
  PosObj (i),(rnd 80-40),(rnd 80-40),(rnd 1500+1300)
ShowMessage "Hopefully, you enjoyed this VR tutorial!"
Wait ContinueButton
ShowMessage "Click 'Continue', then watch the sun for a surprise. Right button to end."
Wait ContinueButton
HideMessage
for i=2 to molecules+1
  RotateObj (i),X=-90
  AutoRotateObj (i),Z=(rnd 5-2.5)
  AutoMoveObj (i),Z=-(rnd 1.5+1.5)
# Keep Klingon warbirds on screen
while 1
  for i=2 to molecules+1
    x,y,z=PosObj (i)
    if z<-1300
      MoveObj (i),Z=2600
    if RightButton
      break 2
    Wait 1
 
# MAKE SURE THAT ONLY THE SKYSPHERE IS PRESENT
# ============================================
def StartSection
  global Android
  # Hidden command to disable undo/redo during movies, which speeds things up
  LeaveTab
  if Android
    Antialias 0
  # Delete everything we don't need
  DelObj !skysphere
  DelImage all
  DelView !Main
  if not Objects
    ShowOriSkySphere
  NumberObj all,1
  HideMessage
  StickRadius 60%
  Boundary wall
  SurfPar Probe=1.4,Resolution=3,Molecular=Numeric,ESPMax=200.0,SmoothCut=Yes,Radii=Solvation
  ColorSurfAll OutCol=AtomCol,OutAlpha=85,InCol=Green,InAlpha=60,Specular=Yes
  UserInput on

# SHOW A TUTORIAL
# ===============
def ShowTutorial textcom()
  # Delete leftovers
  UserInput On
  # Hidden command to keep undo/redo active during movies, helps in case of building accidents
  KeepTab
  # Show the messages
  for i=1 to count textcom step 3
    ShowMessage (textcom(i))
    # textcom(i+1) IS -1 TO EXECUTE ALWAYS, -2 TO EXECUTE ONLY IF NO CELL IS PRESENT, -3 NOT TO WAIT, -4 TO ADD A SKY SPHERE
    if textcom(i+1)!=-3
      Wait ContinueButton
    cellobj = ListObj SimCell
    if textcom(i+1)==Atoms or textcom(i+1)==-1 or (textcom(i+1)==-2 and !cellobj) or textcom(i+1)<-2
      # Number of atoms unchanged, assume that user just clicked 'Continue', or always execute
      (textcom(i+2))
    if textcom(i+1)==-4
      # We loaded a scene, which deletes the skysphere
      ShowOriSkySphere
    if textcom(i+1)==-5
      # Switch to faster molecular surface for interactive dihedral changes+minimization
      SurfPar Molecular=Gaussian
    if textcom(i+1)==-6
      # Prepare minimization
      TempCtrl SteepDes
      Sim On
      Wait SpeedMax<3000
      Sim Off  
      
      
# LOAD AND ORIENTATE SKYSPHERE
# ============================
def ShowOriSkySphere
  obj = ShowSkySphere space061.xei
  PosOriObj (obj),0,0,600,0,180,0
          
    
  
  
